#!/usr/bin/env node
var branches;
var buffer;
var Events = require('events').EventEmitter;
var ee = new Events();
var emit = ee.emit;
var exec = require('child_process').exec;
var on = ee.on;
var prompt = require('prompt');
var Q = require('q');
var regEx;

_initializeEvents();

if(process.argv.length < 3) {
  console.error('Must provide an expression.');
  process.exit(1);
}

regEx = process.argv[2];

emit('get-buffer', regEx);

function _askToProceed() {
  var defer = Q.defer();

  branches.map(function mapBranches(branch) {
    console.log('- ' + branch);
  });

  console.log('==============================');
  console.log('Would you like to delete the selected branches?');

  prompt.start();

  prompt.get({
    properties: [{
        name: 'yesOrNo',
        description: '(y/n)',
        pattern: /^(y|n)$/,
        message: 'Only y or n are valid answers',
        required: true
      }]
  }, function proceedCallback(err, results) {
    if(results[0] === 'y') {
      defer.resolve();
    } else {
      defer.reject();
      console.warn('Exiting...');
      process.exit(0);
    }
  });

  return defer.promise;
}

function _getBuffer(regEx) {
  exec('git branch', function bufferCallback(err, buffer, stderr) {
    if(_isError(err)) {
      emit('get-buffer:error', err);
    } else {
      emit('parse-buffer', buffer);
    }
  });
}

function _isError(err) {
  return (err && err.code === 1);
}

function _parseBuffer(buffer) {
  try {
    var exp = new RegExp(regEx);
    var matchedBranches = [];
    var masterIndex;
    var output = buffer.toString();

    output = output.replace(/\n/g, ',');
    output = output.substr(0, output.lastIndexOf(','))
    .replace(/[*] /, '')
    .replace(/\s\s/g, '')
    .split(',');

    masterIndex = output.indexOf('master');

    output.splice(masterIndex, 1);

    for(var i = 0; i < output.length; i++) {
      exp.lastIndex = 0;

      if(!exp.test(output[i])) {
        output.splice(i, 1);
        i--;
      }
    }

    branches = output;

    if(branches.length < 1) {
      emit('nothing-matches:error');
    } else {
      emit('ask-to-proceed', branches);
    }
  } catch(e) {
    emit('parse-buffer:error', e);
  }
}

function _removeBranches(branches) {
  branches.map(function(branch) {
    _removeLocalBranch(branch);
    _removeRemoteBranch(branch);
  });

  function _removeLocalBranch(branch) {
    exec('git branch -D ' + branch, function removeRemoveCallback(err, out) {
      if(_isError(err)) {
        emit('local-branch-removed:error', err, branch);
      } else {
        emit('local-branch-removed:success', branch);
      }
    });
  }

  function _removeRemoteBranch(branch) {
    exec('git push origin :' + branch, function removeRemoveCallback(err, out) {
      if(_isError(err)) {
        emit('remote-branch-removed:error', err, branch);
      } else {
        emit('remote-branch-removed:success', branch);
      }
    });
  }
}

function _switchToMaster(branches) {
  exec('git checkout master', function switchToMaster(err, out) {
    if(_isError(err)) {
      emit('switch-to-master:error', err);
    } else {
      emit('switch-to-master:success', branches);
    }
  });
}

function _initializeEvents() {
  on('get-buffer', function getBufferEvent(regEx) {
    _getBuffer(regEx);
  });

  on('get-buffer:error', function getBufferError(e) {
    console.error('Unable to retrieve response buffer' + '\n' + e.message);
  });

  on('parse-buffer', function parseBufferEvent(buffer) {
    _parseBuffer(buffer);
  });

  on('parse-buffer:error', function parseBufferError(e) {
    console.log('Unable to parse results... exiting.' + '\n' + e.message);
    process.exit(1);
  });

  on('ask-to-proceed', function askToProceedEvent(branches) {
    _askToProceed().then(function askToProceedCallback(response) {
      emit('switch-to-master');
    });
  });

  on('switch-to-master', function switchToMasterEvent() {
    _switchToMaster(branches);
  });

  on('switch-to-master:error', function switchToMasterError(e) {
    emit('try-to-switch-to-develop');
  });

  on('switch-to-master:success', function switchToMasterSuccess(branches) {
    emit('remove-branches', branches);
  });

  on('try-to-switch-to-develop', function tryDevEvent() {
    exec('git checkout develop', function(err, out) {
      if(_isError(err)) {
        emit('try-to-switch-to-develop:error', err);
      }
    });
  });

  on('try-to-switch-to-develop:error', function tryDevError(e) {
    console.log('Failed to switch to master or dev branch... exiting.'+ '\n' + e.message);
    process.exit(1);
  });

  on('remove-branches', function removeBranchesEvent(branches) {
    _removeBranches(branches);
  });

  on('local-branch-removed:success', function removeLocalSuccessEvent(branch) {
    console.log('Local branch removed: ' + branch);
  });

  on('local-branch-removed:error', function removeLocalErrorEvent(e, branch) {
    console.log('Failed to remove local branch: ' + branch+ '\n');
  });

  on('remote-branch-removed:success', function removeRemoteSuccessEvent(branch) {
    console.log('Remote branch removed: ' + branch);
  });

  on('remote-branch-removed:error', function removeRemoteSuccessError(e, branch) {
    console.log('Failed to remove remote branch: ' + branch + '\n');
  });

  on('nothing-matches:error', function nothingMatchesError() {
    console.log('Nothing seems to match the provided expression.');
  });
}
